<?php
/*	Modul: alkisbestnw.php

	ALKIS-Buchauskunft, Kommunales Rechenzentrum Minden-Ravensberg/Lippe (Lemgo).
	Bestandsnachweis für ein Grundbuch aus ALKIS PostNAS

	Version:
	2016-02-24 Version fuer norGIS-ALKIS-Import
	2016-11-29 HTML5, Gemeinsam genutzte Datenbanken ermöglichen, Markierung der Sortierung
	2017-06-01 Function "eigentuemer" ohne Parameter $con
	2017-06-08 Validierung Eingabeparameter, Farbwechsel der Tabelle je Grundstück, Feinschliff und HTML-Profilierung
	2017-10-09 Schlüssel der Blattart bei berechtigten Blättern, sauber filtern auf endet IS NULL
	2018-04-23 Blatt und Eigentümer in eine Tabelle.
	2018-05-03 Anpassung an Grundstücksnachweis, kein Export für fiktives Blatt
	2018-11-09 Umstellung Full-Schema
	2018-11-14 Korrektur zu Full-Schema
*/
session_start();
$showkey="n"; $nodebug=""; // Var. initalisieren
$cntget = extract($_GET); // Parameter in Variable umwandeln

// strikte Validierung aller Parameter
if (!isset($gmlid) or !preg_match('#^[0-9A-Za-z]{16}$#', $gmlid)) {die("Eingabefehler gmlid");}
if (!isset($gkz) or !preg_match('#^[0-9]{3}$#', $gkz)) {die("Eingabefehler gkz");}
if (!preg_match('#^[j|n]{0,1}$#', $showkey)) {die ("Eingabefehler showkey");}
if ($showkey == "j") {$showkey=true;} else {$showkey=false;}
if (!preg_match('#^j{0,1}$#', $nodebug)) {die("Eingabefehler nodebug");}

require_once("alkis_conf_location.php");
if ($auth == "mapbender") {require_once($mapbender);}
include("alkisfkt.php");
?>
<!doctype html>
<html lang="de">
<head>
	<meta charset="utf-8">
	<meta name="viewport" content="width=device-width, initial-scale=1.0">
	<title>ALKIS Bestandsnachweis</title>
	<link rel="stylesheet" type="text/css" href="alkisauszug.css">
	<link rel="shortcut icon" type="image/x-icon" href="ico/Grundbuch.ico">
	<script type="text/javascript">
		function ALKISexport(gmlid) {
			window.open('alkisexport.php?gkz=<?php echo $gkz;?>&tabtyp=grundbuch&gmlid=' + gmlid);
		}
	</script>
</head>
<body>
<?php
// Mit URL-Parameter "&nodebug=j" kann man die Entwicklungsumgebung (in Conf definiert) ausschalten.
// Manuell an die URL anfügen, wirkt nur temporär und wird nicht in Links weiter gereicht.
if ($nodebug == "j") {$debug=0;} 

$con = pg_connect($dbconn." options='--application_name=ALKIS-Auskunft_alkisbestnw.php'");
if (!$con) echo "\n<p class='err'>Fehler beim Verbinden der DB</p>";

// G R U N D B U C H
$sql="SELECT g.gml_id, g.bezirk, g.buchungsblattnummermitbuchstabenerweiterung AS blatt, g.blattart, wb.beschreibung AS blattartv, wb.dokumentation AS blattartd, 
b.gml_id, b.bezirk, b.bezeichnung AS beznam, d.gml_id, d.land, d.bezeichnung, d.stelle, d.stellenart, wd.beschreibung AS stellev FROM ax_buchungsblatt g 
LEFT JOIN ax_buchungsblattbezirk b ON g.land=b.land AND g.bezirk=b.bezirk 
LEFT JOIN ax_dienststelle d ON b.land=d.land AND b.gehoertzu_stelle=d.stelle 
LEFT JOIN ax_blattart_buchungsblatt wb ON g.blattart = wb.wert
LEFT JOIN ax_behoerde wd ON d.stellenart = wd.wert
WHERE g.gml_id= $1 AND g.endet IS NULL AND b.endet IS NULL AND d.endet IS NULL;";
// .. AND d.stellenart=1000 

$v = array($gmlid);
$res = pg_prepare("", $sql);
$res = pg_execute("", $v);
if (!$res) {
	echo "\n<p class='err'>Fehler bei Grundbuchdaten.</p>";
	if ($debug > 2) {echo "\n<p class='dbg'>SQL=<br>".htmlentities($sql, ENT_QUOTES, "UTF-8")."<br>$1 = gml_id = '".$gmlid."'</p>";}
}
if ($row = pg_fetch_assoc($res)) {
	$blattkey=$row["blattart"];
	$blattart=$row["blattartv"];
	$blatt=ltrim($row["blatt"], "0");

	echo "<p class='gbkennz'>ALKIS Bestand ".$row["bezirk"]." - ".$blatt."&nbsp;</p>"; // Balken
	echo "\n<h2>Grundbuch</h2>";
	echo "\n<table class='outer'>" // Blatt UND Eigent.
		."\n<tr>\n\t<td class='ll'><img src='ico/Grundbuch.png' width='16' height='16' alt=''> Blatt:</td>"  // outer Links
		."\n\t<td colspan='2'>"; // Outer Mitte, Kennz. im Rahmen
		if ($blattkey == 1000) {
			echo "\n\t\t<table class='kennzgb' title='Bestandskennzeichen'>";
		} else {
			echo "\n\t\t<table class='kennzgbf' title='Bestandskennzeichen'>"; // dotted
		}
			echo "\n\t\t<tr>"
				."\n\t\t\t<td class='head'>"; // .d nicht gefüllt
				if ($showkey) {echo "<span class='key'>".$row["stellenart"]."&nbsp;</span>";}
				echo $row["stellev"]."</td>";
				echo "\n\t\t\t<td class='head'>Bezirk</td>"
				."\n\t\t\t<td class='head' title='".htmlentities($row["blattartd"], ENT_QUOTES, "UTF-8")."'>";
				if ($showkey) {echo "<span class='key'>".$blattkey."&nbsp;</span>";}
				echo $blattart."</td>"
			."\n\t\t</tr>\n\t\t<tr>"
				."\n\t\t\t<td title='Amtsgerichtsbezirk'>";
				if ($showkey) {echo "<span class='key'>".$row["stelle"]."</span><br>";}
				echo htmlentities($row["bezeichnung"], ENT_QUOTES, "UTF-8")."</td>"
				."\n\t\t\t<td title='Grundbuchbezirk'>";
				if ($showkey) {echo "<span class='key'>".$row["bezirk"]."</span><br>";}
				echo htmlentities($row["beznam"], ENT_QUOTES, "UTF-8")."</td>"
				."\n\t\t\t<td title='Grundbuch-Blatt'><span class='wichtig'>".$blatt."</span></td>"
			."\n\t\t</tr>"
		."\n\t\t</table>"
		."\n\t</td>\n\t<td>&nbsp;"
	."\n\t</td>\n</tr>";
}

if ($blattkey == 5000) { // fikt. Blatt
	echo "\n<p>Keine Angaben zum Eigentum bei fiktivem Blatt.</p>";
} else { // E I G E N T U E M E R
	$n = eigentuemer($gmlid, true, ""); // MIT Adressen.
	if ($n == 0) { // keine NamensNr, kein Eigentuemer
		echo "\n<p class='err'>Keine Namensnummer gefunden.</p>"
		."\n<p>Bezirk: ".$row["bezirk"].", Blatt: ".$blatt.", Blattart ".$blattkey." (".$blattart.")</p>";
	}
}
echo "\n</table>";

// Vorab pruefen, ob der Fall "Rechte an .." vorliegt.
// ToDo: +++ Es gibt auch den Fall, dass auf der nächsten Stufe "nur Rechte" angezeigt werden, noch keine Flurstücke
// ++ Also 3 Möglichkeiten:
// - Nur Rechte an  (+++ FEHLT NOCH ++)
// - Rechte und Flurstücke
// - Flurstücke

// GB-Blatt  <istBestandteilVon<  sh=herrschend  >an/zu>  sd=dienend
$sql="SELECT count(sd.laufendenummer) AS anzahl "
."FROM ax_buchungsstelle sh JOIN ax_buchungsstelle sd ON (sd.gml_id=ANY(sh.an) OR sd.gml_id=ANY(sh.zu)) "
."WHERE sh.istbestandteilvon= $1 AND sd.endet IS NULL AND sh.endet IS NULL;";

$v=array($gmlid); // GB-Blatt
$res=pg_prepare("", $sql);
$res=pg_execute("", $v);
if (!$res) echo "\n<p class='err'>Fehler bei Suche nach Buchungen.</p>";
$row=pg_fetch_assoc($res);
$anz=$row["anzahl"];

echo "\n<hr>\n\n<h3><img src='ico/Flurstueck.png' width='16' height='16' alt=''>";
if ($anz > 0) { // auch "Rechte an"
	echo " Rechte und Flurst&uuml;cke</h3>\n<table class='fs'>";
	// zusaetzliche Kopfzeile "Rechte"
	echo "\n<tr>" 
		."\n\t<td>&nbsp;</td>"
		."\n\t<td class='dien' title='herrschendes Grundst&uuml;ck'>herrschende Buchungsart</td>"
		."\n\t<td class='dien'>Anteil</td>"
		."\n\t<td class='dien'>Bezirk</td>"
		."\n\t<td class='dien'>Blatt</td>"
		."\n\t<td class='dien'>BVNR</td>"
		."\n\t<td class='dien' title='dienendes Grundst&uuml;ck'>Buchungsart</td>"
		."\n\t<td>&nbsp;</td>"
	."\n</tr>";
} else { // keine Rechte an, nur Flurstücke gebucht
	echo " Flurst&uuml;cke</h3>\n<table class='fs'>";
}

// gemeinsame Kopfzeile "Buchung + Flurstück"
echo "\n<tr>"
	."\n\t<td class='heads' title='laufende Nummer Bestandsverzeichnis (BVNR) = Grundst&uuml;ck'><span class='wichtig'>BVNR</span><img src='ico/sort.png' width='10' height='10' alt='' title='prim&auml;re Sortierung'></td>"
	."\n\t<td class='head'>Buchungsart</td>"
	."\n\t<td class='head'>&nbsp;</td>"
	."\n\t<td class='heads'>Gemarkung</td>"
	."\n\t<td class='heads'>Flur</td>"
	."\n\t<td class='heads fsnr' title='Flurst&uuml;cksnummer (Z&auml;hler / Nenner)'><span class='wichtig'>Flurst.</span></td>"
	."\n\t<td class='head flag'>Fl&auml;che</td>"
	."\n\t<td class='head nwlink noprint' title='Verlinkung zu anderen Nachweis-Arten und verbundenen Objekten'>weitere Auskunft</td>"
."\n</tr>";

// Blatt ->  B u c h u n g s s t e l l e
// aktuelles ax_buchungsblatt <istBestandteilVon< ax_buchungsstelle 
$sql ="SELECT s.gml_id, s.buchungsart, s.laufendenummer AS lfd, s.beschreibungdesumfangsderbuchung AS udb, s.zaehler, s.nenner, 
s.nummerimaufteilungsplan AS nrap, s.beschreibungdessondereigentums AS sond, b.beschreibung as bart, b.dokumentation
FROM ax_buchungsstelle s LEFT JOIN ax_buchungsart_buchungsstelle b ON s.buchungsart = b.wert
WHERE s.istbestandteilvon= $1 AND s.endet IS NULL ORDER BY cast(s.laufendenummer AS integer);";
$v=array($gmlid); // Rel. istBestandteilVon
$res=pg_prepare("", $sql);
$res=pg_execute("", $v);
if (!$res) {
	echo "\n<p class='err'>Fehler bei Buchung.</p>";
	if ($debug > 2) {echo "\n<p class='dbg'>SQL=<br>".htmlentities($sql, ENT_QUOTES, "UTF-8")."<br>$1 = gml_id = '". $gmlid."'</p>";}
}
$i=0;
$fscnt=0;
$zpaar=false;
while($row = pg_fetch_assoc($res)) { // Loop Buchungsstelle (Grundstück)
	$lfdnr=$row["lfd"];
	$bvnr=str_pad($lfdnr, 4, "0", STR_PAD_LEFT);
	$gml_bs=$row["gml_id"]; // id der buchungsstelle
	$bartkey=$row["buchungsart"]; // Schlüsselwert
	$bart=$row["bart"]; // BuchungsART Text dazu
	$bartstory=htmlentities($row["dokumentation"], ENT_QUOTES, "UTF-8"); // .. für Anzeige aufbereitet
	if ($row["zaehler"] == "") {
		$anteil = "";
	} else {
		$anteil = $row["zaehler"]."/".$row["nenner"];
	}
	if ($zpaar) {$trclass='paa';} else {$trclass='unp';} // Farbwechsel je Grundstück
	$zpaar=!$zpaar;

	// F l u r s t u e c k s d a t e n  zur direkten Buchungsstelle
	$j = bnw_fsdaten($lfdnr, $gml_bs, $bart, $anteil, true, $bartkey, $bartstory); // return=Anzahl der FS
	$fscnt=$fscnt + $j;
	if ($j == 0) { //  k e i n e  Flurstuecke gefunden (Miteigentumsnteil usw.)
		// Bei "normalen" Grundstuecken wurden hier Flurstuecksdaten gefunden und ausgegeben.
		// Bei Miteigentumsanteil, Erbbaurecht usw. muss nach den "dienenden" Buchungsstellen gesucht werden, darauf sind die FS gebucht.
		//  Buchungsstelle >an/zu> (andere)Buchungsstelle >istBestandTeilVon> "FiktivesBlatt (ohne) Eigentuemer"

		// dann andere "dienende" Buchungsstellen
		//  sh=herrschend          sd=dienend
		//  ax_buchungsstelle >zu> ax_buchungsstelle (des gleichen Blattes)
		//  ax_buchungsstelle >an> ax_buchungsstelle (anderes Blatt, z.B Erbbaurecht an)

		// Mit "UNION" um die Fälle mit JOIN über die Arrays "an" und "zu" unterscheiden zu können
		$sql="SELECT uni.*, wb.beschreibung AS bart FROM "
		."(SELECT sda.gml_id, sda.buchungsart, sda.laufendenummer AS lfd, sda.beschreibungdesumfangsderbuchung AS udb, sda.nummerimaufteilungsplan AS nrap, sda.beschreibungdessondereigentums AS sond, 'an' as anzu "
		."FROM ax_buchungsstelle sha JOIN ax_buchungsstelle sda ON sda.gml_id=ANY(sha.an) "
		."WHERE sha.gml_id= $1 AND sha.endet IS NULL AND sda.endet IS NULL "
		."UNION SELECT sdz.gml_id, sdz.buchungsart, sdz.laufendenummer AS lfd, sdz.beschreibungdesumfangsderbuchung AS udb, sdz.nummerimaufteilungsplan AS nrap, sdz.beschreibungdessondereigentums AS sond, 'zu' as anzu "
		."FROM ax_buchungsstelle shz JOIN ax_buchungsstelle sdz ON sdz.gml_id=ANY(shz.zu) "
		."WHERE shz.gml_id= $1 AND shz.endet IS NULL AND sdz.endet IS NULL "
		.") AS uni LEFT JOIN ax_buchungsart_buchungsstelle wb ON uni.buchungsart = wb.wert "
		."ORDER BY uni.lfd;";
	//	Testfall suchen: SELECT gml_id, zu, durch FROM ax_buchungsstelle WHERE NOT zu IS NULL OR NOT durch IS NULL;
	//	Die Relationen "zu" und "durch" kommen in der Praxis nicht vor.

		$v=array($gml_bs); // gml_id der "herrschenden" Buchungs-Stelle
		$resan=pg_prepare("", $sql); // Suche nach "dienender" Buchungsstelle
		$resan=pg_execute("", $v);
		if (!$resan) {
			echo "\n<p class='err'>Fehler bei 'andere Buchungsstelle'.</p>";
			if ($debug > 2) {echo "\n<p class='dbg'>SQL=<br>".htmlentities($sql, ENT_QUOTES, "UTF-8")."<br>$1 = '".$gml_bs."'</p>";}
		}

		$a=0; // count
		$altbvnr=""; // Gruppenwechsel
		while($rowan= pg_fetch_assoc($resan)) {
			$lfdnran=$rowan["lfd"];	// BVNR an
			$gml_bsan=$rowan["gml_id"];	// id der buchungsstelle an
			$baan=$rowan["bart"];	// Buchungsart an, entschluesselt
			$anzu=$rowan["anzu"];	// Welche Relation? Recht an oder Recht zu ?

			// dienende Buchungsstelle
			$sql ="SELECT b.gml_id, b.land, b.bezirk, b.buchungsblattnummermitbuchstabenerweiterung AS blatt, b.blattart, wb.beschreibung AS blattartv, z.bezeichnung AS beznam "
			."FROM ax_buchungsblatt b JOIN ax_buchungsstelle s ON b.gml_id=s.istbestandteilvon "
			."LEFT JOIN ax_buchungsblattbezirk z ON b.land=z.land AND b.bezirk=z.bezirk "
			."LEFT JOIN ax_blattart_buchungsblatt wb ON b.blattart = wb.wert "
			."WHERE s.gml_id= $1 AND b.endet IS NULL AND s.endet IS NULL AND z.endet IS NULL ORDER BY b.land, b.bezirk, b.buchungsblattnummermitbuchstabenerweiterung;";
			$v=array($gml_bsan);
			$fbres=pg_prepare("", $sql);
			$fbres=pg_execute("", $v);
			if (!$fbres) {
				echo "\n<p class='err'>Fehler bei fiktivem Blatt.</p>";
				if ($debug > 2) {echo "\n<p class='dbg'>SQL=<br>".htmlentities($sql, ENT_QUOTES, "UTF-8")."<br>$1 = gml_id = '".$gmlid."'</p>";}			
			}
			$b=0;
			while($fbrow = pg_fetch_assoc($fbres)) { // genau 1
				$fbgml=$fbrow["gml_id"];
				$fbland=$fbrow["land"];
				$fbbez=$fbrow["bezirk"];
				$beznam=$fbrow["beznam"];
				$fbblatt=ltrim($fbrow["blatt"], "0");
				$fbbartkey=$fbrow["blattart"]; // Key
				$fbbart=$fbrow["blattartv"];  // Value
				$b++;
			}
			if ($b != 1) { // Überwachen: genau 1?
				echo "\n<p class='err'>Anzahl fiktive Bl&auml;tter zu anderer Buchungstelle = ".$b."</p>";
			}
			// Grundbuchdaten zur dienenden Buchungsstelle
			echo "\n<tr class='".$trclass."'>"
				."\n\t<td id='bvnr".$lfdnr."'>"; // Sp.1 Erbbau BVNR, Sprungmarke
					if($bvnr == $altbvnr) {	// gleiches Grundstueck
						echo "&nbsp;"; // dann Anzeige unterdruecken
					} else {
					//	echo "<a id='bvnr".$lfdnr."'></a>" // Sprungmarke
						echo "<span class='wichtig'>".$bvnr."</span>"; 
						$altbvnr = $bvnr; // Gruppenwechsel merken
					}
				echo "</td>"
				."\n\t<td class='dien'>"; // Sp.2 Buchung
					if ($showkey) {echo "<span class='key'>".$row["buchungsart"]."</span> ";}
				echo $bart." an</td>"
				."\n\t<td class='dien'>".$anteil."</td>" // Sp.3 Anteil
				."\n\t<td class='dien' title='Grundbuch-Bezirk'>"; // Sp.4 Gemarkg., hier Bezirk
					if ($showkey) {echo "<span class='key'>".$fbbez."</span> ";}
					echo htmlentities($beznam, ENT_QUOTES, "UTF-8")
				."</td>";
				echo "\n\t<td class='dien' title='".$fbbart."'>".$fbblatt."</td>" // Sp. 5 Blatt
					."\n\t<td class='dien' title='Bestandsverzeichnis-Nummer'>".str_pad($lfdnran, 4, "0", STR_PAD_LEFT)."</td>" // Sp.6 BVNR
					."\n\t<td class='dien' title='Buchungsart'>"; // Sp.7 Buchungsart
					if ($showkey) {echo "<span class='key'>".$rowan["buchungsart"]."</span> ";}
					echo $baan." "
				."</td>"
				."\n\t<td>" // Sp.8 Link ("an" oder "zu"?)
					."\n\t\t<p class='nwlink noprint'>".$anzu; // Recht "an" oder "zu"

					// Buchung
					// +++ <a  gsnachw 

					// Bestand
					if ($showkey) {echo " <span class='key'>".$fbbartkey."</span>";} // Blatt-Art Schlüssel
					echo "\n\t\t\t<a href='alkisbestnw.php?gkz=".$gkz."&amp;gmlid=".$fbgml;
						if ($showkey) {echo "&amp;showkey=j";}
						echo "#bvnr".$lfdnran // Sprungmarke auf der Seite
						."' title='Zum Grundbuchnachweis des dienenden Blattes'>";
						echo $fbbart // Blatt-Art entschlüsselt
					." <img src='ico/GBBlatt_link.png' width='16' height='16' alt=''></a>";

					echo "\n\t\t</p>"
				."\n\t</td>"
			."\n</tr>"; 

			// Flurstuecksdaten zur dienenden Buchungsstelle
			// Buchungsart wird nur in erster Zeile ausgegeben, hier leer
			$aj = bnw_fsdaten($lfdnran, $gml_bsan, "", $anteil, false, "", ""); // return=Anz.FS
			$fscnt=$fscnt + $aj;
			if ($rowan["nrap"] != "") {
				echo "\n<tr>"
					."\n\t<td class='sond' colspan=8>Nr. im Aufteilungsplan: ".$rowan["nrap"]."</td>"
				."\n</tr>";
			}
			if ($rowan["sond"] != "") {
				echo "\n<tr>"
					."\n\t<td class='sond' colspan=8>Verbunden mit dem Sondereigentum an: ".htmlentities($rowan["sond"], ENT_QUOTES, "UTF-8")."</td>"
				."\n</tr>";
			}
			$a++;
		}
		if ($a == 0) {
			echo "\n<tr class='".$trclass."'>"
				."\n\t<td><span class='wichtig'>".$bvnr."</span></td>"
				."\n\t<td colspan=7><p class='warn'>Keine Flurst&uuml;cke zu Buchung ".$bvnr." im berechtigten Bereich.</p></td>"
			."\n</tr>";
		}
	}
	$i++; 
	if ($row["nrap"] != "") { // Nr im Auft.Plan
		echo "\n<tr>"
			."\n\t<td class='nrap' colspan=8>Nummer <span class='wichtig'>".$row["nrap"]."</span> im Aufteilungsplan.</td>"
		."\n</tr>";
	}
	if ($row["sond"] != "") { // Sondereigentumsbeschreibung
		echo "\n<tr>"
			."\n\t<td class='sond' colspan=8>Verbunden mit dem Sondereigentum an: ".$row["sond"]."</td>"
		."\n</tr>";
	} // Ende Buchungsstelle/BVNR
} // Ende Loop
echo "\n</table>";

if ($i == 0) {
	echo "\n<p class='err'>Keine Buchung gefunden.</p>";
} else {
	if ($i > 5 and $fscnt > 5) { // nur wenn nicht auf einen Blick zu erkennen
		echo "\n<p class='cnt'>".$i." Buchungen mit ".$fscnt." Flurst&uuml;cken</p>";
	}
}

// B e r e c h t i g t e  Buchungsblaetter  mit Recht an/zu dem aktuellen (fiktiven?) Blatt

// bf                          sf            sb                               bb
// Blatt   <istBestandteilVon< Stelle  <an<  Stelle      >istBestandteilVon>  Blatt
// Fiktiv                      Fiktiv  <zu<  Berechtigt                       Berechtigt
$sql="SELECT sf.laufendenummer AS anlfdnr, bb.gml_id, bb.land, bb.bezirk, bb.buchungsblattnummermitbuchstabenerweiterung AS blatt, bb.blattart, wa.beschreibung AS blattartv, 
sb.gml_id AS gml_s, sb.laufendenummer AS lfdnr, sb.buchungsart, wb.beschreibung AS bart, wb.dokumentation AS bartd, bz.bezeichnung AS beznam, d.bezeichnung, d.stelle, d.stellenart, wd.beschreibung AS stellev 
FROM ax_buchungsstelle sf JOIN ax_buchungsstelle sb ON (sf.gml_id=ANY(sb.an) OR sf.gml_id=ANY(sb.zu)) 
JOIN ax_buchungsblatt bb ON bb.gml_id=sb.istbestandteilvon 
LEFT JOIN ax_buchungsblattbezirk bz ON bb.land=bz.land AND bb.bezirk=bz.bezirk 
LEFT JOIN ax_dienststelle d ON bz.land=d.land AND bz.gehoertzu_stelle=d.stelle 
LEFT JOIN ax_blattart_buchungsblatt wa ON bb.blattart = wa.wert
LEFT JOIN ax_buchungsart_buchungsstelle wb ON sb.buchungsart = wb.wert
LEFT JOIN ax_behoerde wd ON d.stellenart = wd.wert
WHERE sf.istbestandteilvon = $1 AND sf.endet IS NULL AND sb.endet IS NULL AND bb.endet IS NULL AND bz.endet IS NULL AND d.endet IS NULL 
ORDER BY cast(sf.laufendenummer AS integer), bz.bezeichnung, bb.buchungsblattnummermitbuchstabenerweiterung, cast(sb.laufendenummer AS integer);";

$v = array($gmlid);
$resb = pg_prepare("", $sql);
$resb = pg_execute("", $v);
if (!$resb) {
	echo "\n<p class='err'>Fehler bei 'Berechtigte Bl&auml;tter.</p>";
	if ($debug > 2) {echo "\n<p class='dbg'>SQL=<br>".htmlentities($sql, ENT_QUOTES, "UTF-8")."<br>$1 = gml_id = '".$gmlid."'</p>";}
}
$b=0; // count: Buchungen / Blätter
$zpaar=false;
while($rowb = pg_fetch_assoc($resb)) {
	if ($b == 0) { // Ueberschrift und Tabelle nur ausgeben, wenn etwas gefunden wurde
		echo "\n\n<h3><img src='ico/Grundbuch_zu.png' width='16' height='16' alt=''> Berechtigte Grundb&uuml;cher</h3>"
		."\n<table class='outer'>"
		."\n<tr>"
			."\n\t<td class='heads' title='lfd. Nr. auf diesem Blatt, wie im Teil Flurst&uuml;cke'><span class='wichtig'>an BVNR</span><img src='ico/sort.png' width='10' height='10' alt='' title='prim&auml;re Sortierung'></td>"
			."\n\t<td class='head'>Dienststelle</td>"
			."\n\t<td class='heads'>Bezirk</td>"
			."\n\t<td class='heads'><span class='wichtig'>Blatt</span></td>"
			."\n\t<td class='heads'>BVNR</td>"
			."\n\t<td class='head'>Buchungsart</td>"
			."\n\t<td class='head nwlink noprint'>Weitere Auskunft</td>"
		."\n</tr>";
	}

	$anlfdnr=$rowb["anlfdnr"];	// an BVNR
	$anlfdnr0=str_pad($anlfdnr, 4, "0", STR_PAD_LEFT); // mit führ.0
	$gml_b=$rowb["gml_id"];		// id des ber. Blattes
	$gml_s=$rowb["gml_s"];		// id der ber. B-Stelle
	$blart=$rowb["blattart"];

	$buch=$rowb["buchungsart"];	// Buchungsart Stelle berechtigt
	$bart=$rowb["bart"];		// BA entschl.
	$lfdnr=$rowb["lfdnr"];		// BVNR ber.
	$blatt=ltrim($rowb["blatt"], "0");
	$bvnr=str_pad($lfdnr, 4, "0", STR_PAD_LEFT);

	if ($zpaar) {$trclass='paa';} else {$trclass='unp';} // Farbwechsel je Zeile = Grundstück
	$zpaar=!$zpaar;

	echo "\n<tr class='".$trclass."'>"; // Der Teil "berechtigte Grundbücher" ist sortiert wie oberer Teil "Flurstücke"
		echo "\n\t<td><span class='wichtig'>".$anlfdnr0."</span></td>"
		."\n\t<td>"; // Amtsgericht,Grundbuchamt
			echo htmlentities($rowb["stellev"], ENT_QUOTES, "UTF-8")." ";
			if ($showkey) {echo "<span class='key'>".$rowb["stelle"]."</span> ";}
			echo htmlentities($rowb["bezeichnung"], ENT_QUOTES, "UTF-8")
		."</td>"
		."\n\t<td>";
			if ($showkey) {echo "<span class='key'>".$rowb["bezirk"]."</span> ";}
			echo htmlentities($rowb["beznam"], ENT_QUOTES, "UTF-8")
		."</td>"
		."\n\t<td><span class='wichtig'>".$blatt."</span></td>"
		."\n\t<td>".$bvnr."</td>"
		."\n\t<td title='".htmlentities($rowb["bartd"], ENT_QUOTES, "UTF-8")."'>";
			if ($showkey) {echo "<span class='key'>".$buch."</span> ";}
			echo $bart
		."</td>"
		."\n\t<td>"
			."\n\t\t<p class='nwlink noprint'>";
			// Bestand
			if ($showkey) {echo "<span class='key'>".$blart."</span> ";}
			echo "\n\t\t\t<a href='alkisbestnw.php?gkz=".$gkz."&amp;gmlid=".$gml_b;
				if ($showkey) {echo "&amp;showkey=j";}
				echo "#bvnr".$lfdnr."' title='Nachweis des berechtigten Blattes an einer Buchung auf ".$blattart."'>".$rowb["blattartv"]; // "an" ggf. durch "zu" ersetzen?
				echo " \n\t\t\t<img src='ico/GBBlatt_link.png' width='16' height='16' alt=''></a>";
			echo "\n\t\t</p>"
		."</td>"
	."\n</tr>";
	$b++;
}
if ($b == 0) {
	if ($blattkey > 2000 ) { // Warnung nicht bei Grundbuchblatt 1000 und Katasterblatt 2000
		echo "\n<p class='err'>Keine berechtigten Bl&auml;tter zu ".$blattart." (".$blattkey.") gefunden.</p>";
	}
} else {
	echo "\n</table>";
	if ($i > 1) {
		echo "\n<p class='cnt'>Rechte anderer Buchungsstellen an ".$b." der ".$i." Buchungen</p>";
	}
}

echo "\n<div class='buttonbereich noprint'><hr>"
	."\n\t<a title='zur&uuml;ck' href='javascript:history.back()'><img src='ico/zurueck.png' width='16' height='16' alt='zur&uuml;ck'></a>&nbsp;"
	."\n\t<a title='Drucken' href='javascript:window.print()'><img src='ico/print.png' width='16' height='16' alt='Drucken'></a>&nbsp;";
if ($blattkey == 5000) { // Nicht bei "fiktives Blatt"
	echo "\n\t<img src='ico/download_gb_no.png' width='32' height='16' alt='Export' title='F&uuml;r ein fiktives Blatt ohne Eigent&uuml;mer ist ein CSV-Export nicht sinnvoll.'>"; 
} else {
	echo "\n\t<a title='Export als CSV' href='javascript:ALKISexport(\"".$gmlid."\")'><img src='ico/download_gb.png' width='32' height='16' alt='Export'></a>";
}
echo "&nbsp;\n</div>";
footer($gmlid, $_SERVER['PHP_SELF']."?", "");
?>
</body>
</html>