<?php
/*	alkisgebaeudenw.php - Gebaeudenachweis
	ALKIS-Buchauskunft, Kommunales Rechenzentrum Minden-Ravensberg/Lippe (Lemgo).

	Version:
	2016-02-24 Version fuer norGIS-ALKIS-Import
	2016-11-29 HTML5, Gemeinsam genutzte Datenbanken ermöglichen
	2016-12-05 Array-Behandlung ax_gebaeude.name
	2017-06-08 Validierung Eingabeparameter, Feinkorrekturen
	2017-06-19 Tool-Tip Gebäudefunktion
	2018-04-23 Kennzeichen eingerückt, &nodebug
	2018-11-09 Umstellung Full-Schema
	2018-11-21 Korrektur LEFT Join Bauweise
*/
session_start();
$showkey="n"; $nodebug=""; // Var. aus Parameter initalisieren
$cntget = extract($_GET); // Parameter in Variable umwandeln

// strikte Validierung aller Parameter
if (!isset($gmlid) or !preg_match('#^[0-9A-Za-z]{16}$#', $gmlid)) {die("Eingabefehler gmlid");}
if (!isset($gkz) or !preg_match('#^[0-9]{3}$#', $gkz)) {die("Eingabefehler gkz");}
if (!preg_match('#^[j|n]{0,1}$#', $showkey)) {die ("Eingabefehler showkey");}
if ($showkey == "j") {$showkey=true;} else {$showkey=false;}
if (!preg_match('#^j{0,1}$#', $nodebug)) {die("Eingabefehler nodebug");}

require_once("alkis_conf_location.php");
if ($auth == "mapbender") {require_once($mapbender);}
include("alkisfkt.php");
echo <<<END
<!doctype html>
<html lang="de">
<head>
	<meta charset="utf-8">
	<meta name="viewport" content="width=device-width, initial-scale=1.0">
	<title>ALKIS Geb&auml;udenachweis</title>
	<link rel="stylesheet" type="text/css" href="alkisauszug.css">
	<link rel="shortcut icon" type="image/x-icon" href="ico/Haus.ico">
</head>
<body>
END;
// Mit URL-Parameter "&nodebug=j" kann man die Entwicklungsumgebung (in Conf definiert) ausschalten.
// Manuell an die URL anfügen, wirkt nur temporär und wird nicht in Links weiter gereicht.
if ($nodebug == "j") {$debug=0;} 

$con = pg_connect($dbconn." options='--application_name=ALKIS-Auskunft_alkisgebaeudenw.php'");
if (!$con) echo "\n<p class='err'>Fehler beim Verbinden der DB</p>";

// Flurstueck
$sqlf="SELECT f.flurnummer, f.zaehler, f.nenner, f.amtlicheflaeche, g.gemarkungsnummer, g.bezeichnung 
FROM ax_flurstueck f LEFT JOIN ax_gemarkung g ON f.land=g.land AND f.gemarkungsnummer=g.gemarkungsnummer 
WHERE f.gml_id= $1 AND f.endet IS NULL AND g.endet IS NULL;";
$v=array($gmlid);
$resf=pg_prepare("", $sqlf);
$resf=pg_execute("", $v);
if (!$resf) {
	echo "\n<p class='err'>Fehler bei Flurst&uuml;cksdaten.</p>";
	if ($debug > 2) {echo "\n<p class='err'>SQL=<br>".$sqlf."<br>$1 = gml_id = '".$gmlid."'</p>";}
}
if ($rowf = pg_fetch_assoc($resf)) {
	$gemkname=htmlentities($rowf["bezeichnung"], ENT_QUOTES, "UTF-8");
	$gmkgnr=$rowf["gemarkungsnummer"];
	$flurnummer=$rowf["flurnummer"];
	$flstnummer=$rowf["zaehler"];
	$nenner=$rowf["nenner"];
	if ($nenner > 0) { // BruchNr
		$flstnummer.="/".$nenner;
	} 
	$flstflaeche = $rowf["amtlicheflaeche"] ;
} else {
	echo "\n<p class='err'>Fehler! Kein Treffer fuer gml_id=".$gmlid."</p>";
}
echo "<p class='geb'>ALKIS Flurst&uuml;ck (Geb&auml;ude) ".$gmkgnr."-".$flurnummer."-".$flstnummer."&nbsp;</p>"; // Balken
echo "\n<h2>Flurst&uuml;ck (Geb&auml;ude)</h2>";
echo "\n<table class='outer'>" // Kopf
	."\n<tr>"
		."\n\t<td>"
		."\n\t<td class='ll'><img src='ico/Flurstueck.png' width='16' height='16' alt=''> Kennzeichen:</td>"
		."\n\t<td>"
			."\n\t\t<table class='kennzfs' title='Flurst&uuml;ckskennzeichen'>" // Kennzeichen in Rahmen
				."\n\t\t<tr>"
					."\n\t\t\t<td class='head'>Gemarkung</td>"
					."\n\t\t\t<td class='head'>Flur</td>"
					."\n\t\t\t<td class='head'>Flurst-Nr.</td>"
				."\n\t\t</tr>\n\t\t<tr>"
					."\n\t\t\t<td title='Gemarkung'>";
					if  ($showkey) {echo "<span class='key'>".$gmkgnr."</span><br>";}
					echo $gemkname."&nbsp;</td>"
					."\n\t\t\t<td title='Flurnummer'>".$flurnummer."</td>"
					."\n\t\t\t<td title='Flurst&uuml;cksnummer (Z&auml;hler / Nenner)'><span class='wichtig'>".$flstnummer."</span></td>"
				."\n\t\t</tr>"
			."\n\t\t</table>"
		."\n\t</td>"
		."\n\t<td>"
			."\n\t\t<p class='nwlink noprint'>" // Links zu anderem Nachweis
				."\n\t\t\t<a href='alkisfsnw.php?gkz=".$gkz."&amp;gmlid=".$gmlid;
				if ($showkey) {echo "&amp;showkey=j";}
				echo "&amp;eig=n' title='Flurst&uuml;cksnachweis'>Flurst&uuml;ck <img src='ico/Flurstueck_Link.png' width='16' height='16' alt=''></a>"
			."\n\t\t</p>"
		."\n\t</td>"
	."\n</tr>"
."\n</table>";

echo "\n<p class='fsd'>Flurst&uuml;cksfl&auml;che: <b>".number_format($flstflaeche,0,",",".") . " m&#178;</b></p>";
pg_free_result($resf);

echo "\n\n<h3><img src='ico/Haus.png' width='16' height='16' alt=''> Geb&auml;ude</h3>"
."\n<p>.. auf oder an dem Flurst&uuml;ck. Ermittelt durch Verschneidung der Geometrie.</p>";

// G e b a e u d e
$sqlg ="SELECT g.gml_id, g.name, g.bauweise, g.gebaeudefunktion, h.beschreibung AS bauweise_beschreibung, u.beschreibung AS bezeichner, u.dokumentation AS gfktd, g.zustand, z.beschreibung AS bzustand, 
round(st_area(g.wkb_geometry)::numeric,2) AS gebflae, "; // GEB-Flaeche komplett auch die Fl. ausserhalb des FS
$sqlg.="round(st_area(ST_Intersection(g.wkb_geometry,f.wkb_geometry))::numeric,2) AS schnittflae, "; // wie viel vom GEB liegt im FS?
$sqlg.="st_within(g.wkb_geometry,f.wkb_geometry) as drin "; // liegt das GEB komplett im FS?
$sqlg.="FROM ax_flurstueck f, ax_gebaeude g 
LEFT JOIN ax_bauweise_gebaeude h ON g.bauweise = h.wert
LEFT JOIN ax_gebaeudefunktion u ON g.gebaeudefunktion = u.wert
LEFT JOIN ax_zustand_gebaeude z ON g.zustand = z.wert
WHERE f.gml_id= $1 AND f.endet IS NULL and g.endet IS NULL "; // ID des akt. FS

// "within" -> nur Geb., die komplett im FS liegen. "intersects" -> auch teil-ueberlappende Flst.
$sqlg.="AND st_intersects(g.wkb_geometry,f.wkb_geometry) = true ";
// RLP: keine Relationen zu Nebengebäuden. Auf Qualifizierung verzichten, sonst werden Nebengebäude nicht angezeigt
//$sqlg.="AND (v.beziehungsart='zeigtAuf' OR v.beziehungsart='hat') ";
$sqlg.="ORDER BY schnittflae DESC;";

$v=array($gmlid);
$resg=pg_prepare("", $sqlg);
$resg=pg_execute("", $v);
if (!$resg) {
	echo "\n<p class='err'>Fehler bei Geb&auml;ude-Verschneidung.</p>";
	if ($debug > 2) {echo "\n<p class='err'>SQL=<br>".$sqlg."<br>$1 = gml_id = '".$gmlid."'</p>";}
}
$gebnr=0;
$gebflsum=0;
echo "\n<hr>\n<table class='geb'>";
	echo "\n<tr>" // T-Header
		."\n\t<td class='head' title='ggf. Geb&auml;udename'>Name</td>"
		."\n\t<td class='heads fla' title='Schnittfl&auml;che zwischen Flurst&uuml;ck und Geb&auml;ude'><img src='ico/sortd.png' width='10' height='10' alt='' title='Sortierung (absteigend)'>Fl&auml;che</td>"
		."\n\t<td class='head' title='gesamte Geb&auml;udefl&auml;che, liegt teilweise auf Nachbar-Flurst&uuml;ck'>&nbsp;</td>"
		."\n\t<td class='head' title='Geb&auml;udefunktion ist die zum Zeitpunkt der Erhebung vorherrschend funktionale Bedeutung des Geb&auml;udes'>Funktion</td>"
		."\n\t<td class='head' title='Bauweise ist die Beschreibung der Art der Bauweise'>Bauweise</td>"
		."\n\t<td class='head' title='Zustand beschreibt die Beschaffenheit oder die Betriebsbereitschaft von Geb&auml;ude. Diese Attributart wird nur dann optional gef&uuml;hrt, wenn der Zustand des Geb&auml;udes vom nutzungsf&auml;higen Zustand abweicht.'>Zustand</td>"
		."\n\t<td class='head nwlink' title='Lagebezeichnung mit Stra&szlig;e und Hausnummer'>Lage</td>"
		."\n\t<td class='head nwlink' title='Link zu den kompletten Hausdaten'>Haus</td>"
	."\n</tr>";
	while($rowg = pg_fetch_assoc($resg)) { // T-Body
		$gebnr = $gebnr + 1;
		$ggml=$rowg["gml_id"];
		$gebflsum=$gebflsum + $rowg["schnittflae"];
		$gnam=trim(trim($rowg["name"], "{}"), '"'); // Gebaeude-Name ist ein Array in der DB: '{"A","B"}'
		// Mehrfachbelegung nur theoretisch. Entklammern reicht. Mal mit und mal ohne "" drum!?
		/* SELECT name FROM ax_gebaeude WHERE NOT name IS NULL AND array_length(name, 1) > 1; */
		$gfktk=htmlentities($rowg["gebaeudefunktion"], ENT_QUOTES, "UTF-8");  // Geb.-Funktion Key
		$gfktv=htmlentities($rowg["bezeichner"], ENT_QUOTES, "UTF-8"); // Geb.Funktion Value
		$gfktd=htmlentities($rowg["gfktd"], ENT_QUOTES, "UTF-8"); // Gebäude-Funktion-Description
		$gzus=htmlentities($rowg["zustand"], ENT_QUOTES, "UTF-8");
		$gzustand=htmlentities($rowg["bzustand"], ENT_QUOTES, "UTF-8");

		// 3 Fälle unterscheiden:
		if ($rowg["drin"] == "t") { // Gebäude liegt komplett in Flurstück
			$f1=$rowg["schnittflae"]." m&#178;";
			$f2="&nbsp;";
			$gstyle="gin"; // siehe .css
		} else {
	       	if ($rowg["schnittflae"] == "0.00") { // Gebäude angrenzend (Grenzbebauung)
				$gstyle="gan";
				$f1="&nbsp;";
				$f2="angrenzend";
			} else { // Teile des Gebäudes stehen auf dem Flurstück
				$gstyle="gtl";
				$f1=$rowg["schnittflae"]." m&#178;";
				$f2="(von ".$rowg["gebflae"]." m&#178;)";
			}
		}

		echo "\n<tr>"
			."\n\t<td class='".$gstyle."'>"; // Sp. 1 Name
			if ($gnam != "") {echo "<span title='Geb&auml;udename'>".$gnam."</span><br>";}
			echo "</td>";

			echo "\n\t<td class='fla'>".$f1."</td>" // Sp. 2
			."\n\t<td class='".$gstyle."'>".$f2."</td>"; // Flächenangaben

			echo "\n\t<td class='".$gstyle."' title='".$gfktd."'>"; // Sp. 3 Funktion
			if ($showkey) {echo "<span class='key'>".$gfktk."</span>&nbsp;";}
			echo $gfktv."</td>";

			echo "\n\t<td class='".$gstyle."'>";
			if ($showkey) {echo "<span class='key'>".$rowg["bauweise"]."</span>&nbsp;";}
			echo $rowg["bauweise_beschreibung"]."&nbsp;</td>";

			echo "\n\t<td class='".$gstyle."'>";
			if ($showkey) {echo "<span class='key'>".$gzus."</span>&nbsp;";}
			echo $gzustand."&nbsp;</td>";

			// 2 Spalten mit Links zu anderen Nachweisen: 1. Lage, 2. Gebäude
			echo "\n\t<td class='nwlink noprint'>"; // Link Lage

			// Zu EINEM Gebäude mehrere Lagebezeichnungen mit Haus- oder Pseudo-Nummer möglich, alle in ein TD 
			// HAUPTgebäude  Geb >zeigtAuf> lage (mehrere)
			$sqll="SELECT 'm' AS ltyp, l.gml_id AS lgml, s.lage, s.bezeichnung, l.hausnummer, '' AS laufendenummer "
			."FROM ax_gebaeude g JOIN ax_lagebezeichnungmithausnummer l ON l.gml_id=ANY(g.zeigtauf) "
			."JOIN ax_lagebezeichnungkatalogeintrag s ON l.kreis=s.kreis AND l.gemeinde=s.gemeinde AND l.lage=s.lage "
			."WHERE g.gml_id= $1 AND g.endet IS NULL AND l.endet IS NULL AND s.endet IS NULL ";

			// UNION - oder NEBENgebäude  Geb >hat> Pseudo
			$sqll.="UNION SELECT 'p' AS ltyp, l.gml_id AS lgml, s.lage, s.bezeichnung, l.pseudonummer AS hausnummer, l.laufendenummer "
			."FROM ax_gebaeude g JOIN ax_lagebezeichnungmitpseudonummer l ON l.gml_id=g.hat "
			."JOIN ax_lagebezeichnungkatalogeintrag s ON l.kreis=s.kreis AND l.gemeinde=s.gemeinde AND l.lage=s.lage "
			."WHERE g.gml_id= $1 AND g.endet IS NULL AND l.endet IS NULL AND s.endet IS NULL "; // ID des Hauses"
		
			$sqll.="ORDER BY bezeichnung, hausnummer;";
			$v = array($ggml);
			$resl = pg_prepare("", $sqll);
			$resl = pg_execute("", $v);
			if (!$resl) {
				echo "\n<p class='err'>Fehler bei Lage mit HsNr.</p>";
				if ($debug > 2) {echo "\n<p class='dbg'>SQL=<br>".$sqll."<br>$1 = gml_id = '".$gmlid."'</p>";}
			}
			while($rowl = pg_fetch_assoc($resl)) { // LOOP: Lagezeilen
				$ltyp=$rowl["ltyp"]; // Lagezeilen-Typ
				$skey=$rowl["lage"]; // Str.-Schluessel
				$snam=htmlentities($rowl["bezeichnung"], ENT_QUOTES, "UTF-8"); //-Name
				$hsnr=$rowl["hausnummer"];
				$hlfd=$rowl["laufendenummer"];
				$gmllag=$rowl["lgml"];
				if ($ltyp == "p") {
					$lagetitl="Nebengebäude - Pseudonummer";
					$lagetxt="Nebengeb&auml;ude Nr. ".$hlfd;
				} else {
					$lagetitl="Hauptgebäude - Hausnummer";
					$lagetxt=$snam."&nbsp;".$hsnr;
				}
				echo "\n\t\t<a title='".$lagetitl."' href='alkislage.php?gkz=".$gkz."&amp;gmlid=".$gmllag."&amp;ltyp=".$ltyp;
					if ($showkey) {echo "&amp;showkey=j";}
					echo "'>";
					if ($showkey) {echo "<span class='key'>(".$skey.")</span>&nbsp;";}
				echo $lagetxt."&nbsp;<img src='ico/Lage_mit_Haus.png' width='16' height='16' alt=''></a><br>";
			} // Ende Loop Lage m.H.
            pg_free_result($resl);
			echo "\n\t</td>";

			echo "\n\t<td class='nwlink noprint'>" // Link Haus
				."\n\t\t<a title='Daten zum Geb&auml;ude-Objekt' href='alkishaus.php?gkz=".$gkz."&amp;gmlid=".$ggml;
				if ($showkey) {echo "&amp;showkey=j";}
				echo "'>Haus&nbsp;<img src='ico/Haus.png' width='16' height='16' alt=''></a>"
			."\n\t</td>"
		."\n</tr>";
	} // Ende Gebäude

	// Footer
	if ($gebnr == 0) {
		echo "\n</table>"
		."<p class='err'><br>Keine Geb&auml;ude auf diesem Flurst&uuml;ck.<br>&nbsp;</p>";
		//if ($debug > 2) {echo "\n<p class='err'>SQL=<br>".$sqlg."<br>$1 = gml_id = '".$gmlid."'</p>";}
	} else {
		echo "\n<tr>"
			."\n\t<td>Summe:</td>"
			."\n\t<td class='fla sum' title='von Geb&auml;uden &uuml;berbaute Fl&auml;che des Flurst&uuml;cks'>".number_format($gebflsum,0,",",".")."&nbsp;&nbsp;&nbsp;&nbsp;m&#178;</td>"
			."\n\t<td colspan='6'>&nbsp;</td>"
		."\n</tr>"
	."\n</table>";
	$unbebaut = number_format(($flstflaeche - $gebflsum),0,",",".") . " m&#178;";
	echo "\n<p>\n<br>Flurst&uuml;cksfl&auml;che abz&uuml;glich Geb&auml;udefl&auml;che: <b>".$unbebaut."</b></p>\n<br>";
}
pg_free_result($resg);
?>
 
<div class='buttonbereich noprint'>
	<hr>
	<a title="zur&uuml;ck" href='javascript:history.back()'><img src="ico/zurueck.png" width="16" height="16" alt="zur&uuml;ck"></a>&nbsp;
	<a title="Drucken" href='javascript:window.print()'><img src="ico/print.png" width="16" height="16" alt="Drucken"></a>
</div>

<?php footer($gmlid, $_SERVER['PHP_SELF']."?", ""); ?>

</body>
</html>
