<?php
/*	Modul: alkisgsnw.php

	ALKIS-Buchauskunft, Kommunales Rechenzentrum Minden-Ravensberg/Lippe (Lemgo).
	Grundstücksnachweis fuer ein Grundstück aus ALKIS PostNAS

	Version:
	2018-05-03 Neues Modul "gsnw" abgeleitet aus "fsnw"
	2018-11-09 Umstellung Full-Schema
*/

function get_GS_Daten($gmlid, $gskennz) {
// Daten zur Buchungsstelle (GS = Grundstück) aus der DB lesen.
// Suche wahlweise über die GML-ID oder über das Grundstücks-Kennzeichen (Bezirk-Blatt-LfdNr) aus der URL.
	global $gkz, $showkey, $debug, $defland, $gsbez, $gsblatt, $gslfdnr, $gsbart;

	$sqlgs="SELECT s.gml_id, b.bezirk, b.buchungsblattnummermitbuchstabenerweiterung AS blatt, s.laufendenummer, s.buchungsart "
		."FROM ax_buchungsstelle s JOIN ax_buchungsblatt b ON s.istbestandteilvon=b.gml_id ";
	if ($gskennz == '') { // normaler Link über gmlid
		$sqlgs.="WHERE s.gml_id= $1 ";
		$v=array($gmlid);
	} else { // Kennzeichen "Bezirk-Blatt-BVNR" alternativ zur gml_id
		$arr=explode("-", $gskennz, 4); // zerlegen
		$zbez=trim($arr[0]); // Bezirk
		if (strlen($zbez) == 6) {
			$land=substr($zbez, 0, 2);
			$zbez=substr($zbez, 2, 4);
		} else { // kein schöner Land ..
			$land=$defland; // Default aus config
		}
		$zblatt=$arr[1];
		if (preg_match('#^[0-9]{1,6}$#', $zblatt)) { // Nur numerisch
			$zblatt=str_pad($zblatt, 6, "0", STR_PAD_LEFT)." "; // 6 Nr + 1 blank
		} elseif (preg_match('#^[0-9A-Z]{1,7}$#', $zblatt)) { // +++ A nur in LETZTER Stelle prüfen
			$zblatt=str_pad($zblatt, 7, "0", STR_PAD_LEFT); // 6 Nr + 1 "A", gesamt 7
		} else {
			die("Fehler in Buchungsblatt im Parameter 'gskennz'.");
		}
		$zlfdnr=str_pad($arr[2], 4, "0", STR_PAD_LEFT); // Lfd.-Nr./BVNR
		$sqlgs.="WHERE b.land= $1 AND b.bezirk= $2 AND b.buchungsblattnummermitbuchstabenerweiterung= $3 AND s.laufendenummer= $4 ";
		$v=array($land, $zbez, $zblatt, $zlfdnr);
	}
	// egal ob Suche mit gmlid ODER Kennzeichen
	$sqlgs.="AND b.endet IS NULL AND s.endet IS NULL;";
	$resgs=pg_prepare("", $sqlgs);
	$resgs=pg_execute("", $v);
	if ($rowgs=pg_fetch_assoc($resgs)) {
		$gmlid=$rowgs["gml_id"];
		$gsbez=$rowgs["bezirk"];
		$gsblatt=$rowgs["blatt"];
		$gslfdnr=$rowgs["laufendenummer"];
		$gsbart=$rowgs["buchungsart"];
	} else {
		echo "\n<p class='err'>Fehler! Kein Treffer f&uuml;r Grundst&uuml;ckskennzeichen='".$gskennz."'</p>\n</body>\n</html>";
		return "";
	}
	pg_free_result($resgs);
	return $gmlid;
}

function bs_dienend($gmlid_h) {
// Eine ggf. vorhandene dienende Buchungsstelle suchen und dessen GML_ID liefern.
	global $debug, $gerooted;
	// BS-herrschend (bekannt)  >an[]>  BS-dienend (gesucht)
	$sqld="SELECT d.gml_id FROM ax_buchungsstelle d JOIN ax_buchungsstelle h ON d.gml_id=any(h.an) "
		."WHERE h.gml_id = $1 and d.endet IS NULL AND h.endet IS NULL";
	$v=array($gmlid_h);
	$resd=pg_prepare("", $sqld);
	$resd=pg_execute("", $v);
	$cntd=0;
	while($rowd=pg_fetch_assoc($resd)) {
		$cntd++;
		$gmlid_d=$rowd["gml_id"];
	}
	if ($cntd == 0) { // KEINE dienende BS
		$gmlid_d="";
	} elseif ($cntd > 1) { // Problem: mehrere dienende BS
		$gmlid_d=""; // wennn mehrere (.an=Array[]), dann nicht eindeutig rückführbar.
		$gerooted=false; // Root nicht erreicht
		if ($debug > 2 ) { // Meldung? Auswahl?
			echo "\n<p class='err'>Mehrere 'diendende' Buchungen zur Buchung '".$gmlid_h."'</p>";
		}
	}
	pg_free_result($resd);
	return $gmlid_d;
}

function Back2theRoots($gmlid_h) {
// Eine Buchungsstelle (aus dem Aufruf-Parameter) zurück führen auf die Basis-Buchungsstelle
// Suchen bis Buchungsart="Grundstück" oder ggf. Blattart="fiktives Blatt"
	global $debug;
	$gh=$gmlid_h; // herrschend
	$gd=$gh; // dienend
	$gr=$gd; // root
	//$steps=0; // wie oft?
	while($gd != "") {
		//$steps++;
		$gr=$gd; // sichern
		$gd=bs_dienend($gh); // an?
		$gh=$gd; // fuer weitere Loop
	}
	//$steps=$steps -1; // erfolgreiche Suchen
	return $gr;
}

session_start();
$showkey="n"; $nodebug="";
$cntget=extract($_GET); // Parameter in Variable

// Validierung
if (isset($gmlid)) { // gml der Buchungsstelle (Aufruf)
	if (!preg_match('#^[0-9A-Za-z]{16}$#', $gmlid)) {die("Eingabefehler gmlid");}
	$gskennz='';
} else { // Alternativ
	$gmlid='';
	if (isset($gskennz)) { // llgggg-bbbbbz-nnnn 
		if (!preg_match('#^[0-9\-_/]{8,18}$#', $gskennz)) {die ("Eingabefehler gskennz");}
	} else {
		$gskennz='';
	}
}
if (!isset($gkz) or !preg_match('#^[0-9]{3}$#', $gkz)) {die("Eingabefehler gkz");}
if (!preg_match('#^[j|n]{0,1}$#', $showkey)) {die ("Eingabefehler showkey");}
if ($showkey == "j") {$showkey=true;} else {$showkey=false;}
if (!preg_match('#^j{0,1}$#', $nodebug)) {die("Eingabefehler nodebug");}

require_once("alkis_conf_location.php");
if ($auth == "mapbender") {require_once($mapbender);}
include("alkisfkt.php");
?>
<!doctype html>
<html lang="de">
<head>
	<meta charset="utf-8">
	<meta name="viewport" content="width=device-width, initial-scale=1.0">
	<title>ALKIS Grundst&uuml;cksnachweis</title>
	<link rel="stylesheet" type="text/css" href="alkisauszug.css">
	<link rel="shortcut icon" type="image/x-icon" href="ico/Grundstueck.ico">
	<script>
		function ALKISexport(gmlid) {
			window.open('alkisexport.php?gkz=<?php echo $gkz;?>&tabtyp=buchung&gmlid=' + gmlid);
		}
	</script>
</head>
<body>
<?php
if ($nodebug == "j") {$debug=0;} 
$con = pg_connect($dbconn." options='--application_name=ALKIS-Auskunft_alkisgsnw.php'");
if (!$con) echo "\n<p class='err'>Fehler beim Verbinden der DB</p>";

$gmlid_p=$gmlid; // Wert aus Parameter merken
$gerooted=true; // Auskunft beginnt mit dienendem Grundstück
$gmlid=get_GS_Daten($gmlid, $gskennz); // nachschauen
$gml_anfrd=$gmlid; // ursprüngliche Anforderung aus URL merken
if ($gmlid == "") { // nix gefunden
	die("Kein Treffer");
} else { // Treffer
	if ($gsbart != "1100") { // Buchungsart "Grundstück"
		$gmlid_r=Back2theRoots($gmlid);
		if ($gmlid_r != $gmlid) { // wurde hoch gerückt
			$gmlid=get_GS_Daten($gmlid_r, ""); // damit weiter arbeiten
		}
	}
}
echo "<p class='gskennz'>ALKIS Grundst&uuml;ck ".$gsbez."-".rtrim($gsblatt)."-".$gslfdnr."&nbsp;</p>\n\n<h2>Grundst&uuml;ck</h2>"; // Balken

if ($gerooted) {
	// F l u r s t u e c k e
	echo "\n<h3><img src='ico/Flurstueck.png' width='16' height='16' alt=''> Flurst&uuml;cke</h3>"
		."\n<table class='fs'>";
	echo "\n<tr>" // Kopfzeile
		."\n\t<td class='heads'>Gemarkung</td>"
		."\n\t<td class='heads'>Flur</td>"
		."\n\t<td class='heads fsnr' title='Flurst&uuml;cksnummer (Z&auml;hler / Nenner)'><span class='wichtig'>Flurst.</span></td>"
		."\n\t<td class='head flag'>Fl&auml;che</td>"
		."\n\t<td class='head nwlink noprint' title='Verlinkung zu anderen Nachweis-Arten und verbundenen Objekten'>weitere Auskunft</td>"
	."\n</tr>";
	//+++ Lage oder Nutzung zum FS in eine zus. Spalte oder Zeile?
	//+++ Tabelle hat noch Platz, SQL in der Loop oder Subquery wäre möglich, weil i.d.R. nur wenige FS je GS gebucht sind.

	$sqlfs="SELECT g.gemarkungsnummer, g.bezeichnung, f.gml_id, f.flurnummer, f.zaehler, f.nenner, f.amtlicheflaeche 
	FROM ax_flurstueck f LEFT JOIN ax_gemarkung g ON f.land=g.land AND f.gemarkungsnummer=g.gemarkungsnummer 
	WHERE f.istgebucht = $1 AND f.endet IS NULL AND g.endet IS NULL ";
	if ($filtgem == '' ) { // ungefiltert
		$v=array($gmlid);
	} else {
		$sqlfs.="AND f.gemeindezugehoerigkeit_kreis = $2 AND f.gemeindezugehoerigkeit_gemeinde = $3 "; // Zuständiges Gebiet
		$v=array($gmlid, $filtkreis, $filtgem);
	}
	$sqlfs.="ORDER BY f.gemarkungsnummer, f.flurnummer, f.zaehler, f.nenner;";
	$resfs = pg_prepare("", $sqlfs);
	$resfs = pg_execute("", $v);
	if (!$resfs) {echo "\n<p class='err'>Fehler bei Flurst&uuml;ck</p>";}

	$j=0;
	$zpaar=false;
	while($rowfs = pg_fetch_assoc($resfs)) {
		$flur= $rowfs["flurnummer"];
		$fskenn=$rowfs["zaehler"];
		if ($rowfs["nenner"] != "") {$fskenn.="/".$rowfs["nenner"];}
		$flae=number_format($rowfs["amtlicheflaeche"],0,",",".") . " m&#178;";

		if ($zpaar) {$trclass='paa';} else {$trclass='unp';} // Farbwechsel
		$zpaar=!$zpaar;
		echo "\n<tr class='".$trclass."'>"; // eine Zeile je Flurstueck
			echo "\n\t<td>";
				if ($showkey) {echo "<span class='key'>".$rowfs["gemarkungsnummer"]."</span> ";}
				echo $rowfs["bezeichnung"]
			."</td>"
			."\n\t<td>".$flur."</td>"
			."\n\t<td class='fsnr'><span class='wichtig'>".$fskenn."</span></td>"
			."\n\t<td class='fla'>".$flae."</td>"
			."\n\t<td>\n\t\t<p class='nwlink noprint'>"
				."\n\t\t\t<a href='alkisfsnw.php?gkz=".$gkz."&amp;gmlid=".$rowfs["gml_id"]."&amp;eig=n";
					if ($showkey) {echo "&amp;showkey=j";}
					echo "' title='Flurst&uuml;cksnachweis'>Flurst&uuml;ck "
					."<img src='ico/Flurstueck_Link.png' width='16' height='16' alt=''>"
				."</a>\n\t\t</p>"
			."\n\t</td>"
		."\n</tr>";
		$j++;
	}
	pg_free_result($resfs);
	if ($j == 0) {echo "\n<tr class='unp'>\n\t<td colspan='5'><p class='warn'>Keine Flurst&uuml;cke im berechtigten Bereich.</p></td>\n</tr>";}
	echo "\n</table>";
} else { // konnte nicht bis zur Basis aufsteigen
	echo "<p>Die angeforderte Buchung hat Rechte an mehreren anderen Buchungen."
	."<br>F&uuml;r die Anzeige der Flurst&uuml;cke muss eine dieser Buchungen (Grundst&uuml;ck) gew&auml;hlt werden.</p>";
	// Auswahl anbieten? Sonst über den Link "Bestand" recherchieren. 
}

// Rechte anderer GS an diesem GS
// Dieser Teil ist fast identisch mit "Flurstücksnachweis", Kommentare siehe dort.
$bartgrp="";
$barttypgrp="";
echo "\n<table class='outer'>";
	$stufe=1;
	$gezeigt=buchung_anzg($gmlid, "j", false, "", 2);
	$anzber=ber_bs_zaehl($gmlid);
	$verf_next = array($gmlid);
	while ($anzber > 0 ) {
		$verf_akt=$verf_next;
		$verf_next=array();
 		$stufe++;
		$i=0;
		foreach($verf_akt as $gml_ber_bs) {
			$i++;
			if (ber_bs_zaehl($gml_ber_bs) > 0) {
				$verf_neu=ber_bs_anzg($gml_ber_bs, "j", false, "", $gml_anfrd);
				$anz_neu=count($verf_neu);
				if ($anz_neu > 0) {
					$verf_next=array_merge($verf_next, $verf_neu);
				}
			}
		}
		$anzber=count($verf_next);
	}
echo "\n</table>\n\n";

pg_close($con);
echo <<<END
<div class='buttonbereich noprint'>
<hr>
	<a title="zur&uuml;ck" href='javascript:history.back()'><img src="ico/zurueck.png" width="16" height="16" alt="zur&uuml;ck"></a>&nbsp;
	<a title="Drucken" href='javascript:window.print()'><img src="ico/print.png" width="16" height="16" alt="Drucken"></a>&nbsp;
END;
if ($stufe < 3) {
	echo "\n\t<a title='Export als CSV' href='javascript:ALKISexport(\"".$gmlid."\")'><img src='ico/download_gs.png' width='32' height='16' alt='Export'></a>";
} else { // Export CSV wäre unvollständig bei Buchungen auf mehr als 2 Ebenen. 2 Ebenen werden im View über UNION und einem Zweig mit 2x "an"-Relation abgefangen.
	echo "\n\t<img src='ico/download_gs_no.png' width='32' height='16' alt='Export' title='Komplexe Buchungen über ".$stufe." Ebenen sind nicht als lineare CSV-Datei exportierbar'>";
}
echo "&nbsp;\n</div>";

footer($gmlid, $_SERVER['PHP_SELF']."?", "");
?>
</body>
</html>