<?php
/*	alkisinlaybaurecht.php - Inlay fuer Template: Baurecht
	ALKIS-Buchauskunft, Kommunales Rechenzentrum Minden-Ravensberg/Lippe (Lemgo).
	hnlich alkisbaurecht, aber nur Basisdaten, kein Footer und keine Flurstcks-Verschneidung.

	Version:
	2016-02-24 Version fuer norGIS-ALKIS-Import
	2016-11-29 HTML5, Gemeinsam genutzte Datenbanken ermglichen
	2017-06-08 Validierung Eingabeparameter, Feinkorrekturen
	2017-10-09 Sauber filtern auf endet IS NULL
	2018-11-09 Umstellung Full-Schema
	2018-11-15 Nachkorrektur
*/
session_start();
$cntget = extract($_GET); // Parameter in Variable umwandeln
// strikte Validierung aller Parameter
if (!isset($gmlid) or !preg_match('#^[0-9A-Za-z]{16}$#', $gmlid)) {die("Eingabefehler gmlid");}
if (!isset($gkz) or !preg_match('#^[0-9]{3}$#', $gkz)) {die("Eingabefehler gkz");}

require_once("alkis_conf_location.php");
if ($auth == "mapbender") {require_once($mapbender);}
echo <<<END
<!doctype html>
<html lang="de">
<head>
	<meta charset="utf-8">
	<meta name="viewport" content="width=device-width, initial-scale=1.0">
	<title>ALKIS Bau-, Raum- oder Bodenordnungsrecht</title>
	<link rel="stylesheet" type="text/css" href="alkisauszug.css">
	<link rel="shortcut icon" type="image/x-icon" href="ico/Gericht.ico">
	<script type="text/javascript">
	function imFenster(dieURL) {
		var link = encodeURI(dieURL);
		window.open(link,'','left=30,top=30,width=680,height=800,resizable=yes,menubar=no,toolbar=no,location=no,status=no,scrollbars=yes');
	}
	</script>
</head>
<body>
END;
$con = pg_connect($dbconn." options='--application_name=ALKIS-Auskunft_alkisinlaybaurecht.php'");
if (!$con) echo "\n<p class='err'>Fehler beim Verbinden der DB</p>";

// Keine Spalten, die eine Berechtigungsprfung nach Gemeinde ermglichen:
$sql ="SELECT r.ogc_fid, r.name, r.stelle, r.bezeichnung AS rechtbez, a.beschreibung AS adfbez, d.bezeichnung AS stellbez, round(st_area(r.wkb_geometry)::numeric,0) AS flae 
FROM ax_bauraumoderbodenordnungsrecht r 
LEFT JOIN ax_artderfestlegung_bauraumoderbodenordnungsrecht a ON r.artderfestlegung = a.wert
LEFT JOIN ax_dienststelle d ON r.land=d.land AND r.stelle=d.stelle 
WHERE r.gml_id= $1 AND r.endet IS NULL AND d.endet IS NULL;";

$v = array($gmlid);
$res = pg_prepare("", $sql);
$res = pg_execute("", $v);

if (!$res) {
	echo "\n<p class='err'>Fehler bei Baurecht.</p>";
	if ($debug > 2) {echo "\n<p class='err'>SQL=<br>".$sql."<br>$1 = ".$gmlid."</p>";}
}
echo "\n<h2><img src='ico/Gericht.png' width='16' height='16' alt=''> Bau-, Raum- oder Bodenordnungsrecht</h2>";

if ($row = pg_fetch_assoc($res)) {
	echo "\n<table>"
		."\n<tr>"
			."\n\t<td class='li'>Art der Festlegung:</td>\n\t<td>"
			."<span class='wichtig'>".$row["adfbez"]."</span></td>"
		."\n</tr>";
		$enam=$row["name"];
		if ($enam != "") {
			echo "\n<tr>"
				."\n\t<td class='li'>Eigenname des Gebietes:</td>\n\t<td>".$enam."</td>"
			."\n</tr>";
		}
		echo "\n<tr>"
			."\n\t<td class='li'>Verfahrensnummer:</td>\n\t<td>".$row["rechtbez"]."</td>"
		."\n</tr>";
		$stell=$row["stelle"];
		if ($stell != "") {
			echo "\n<tr>"
				."\n\t<td class='li'>Dienststelle:</td>\n\t<td>".$row["stellbez"]."</td>"
			."\n</tr>";
		}
		echo "\n<tr>"
			."\n\t<td class='li'>Fl&auml;che:</td>";
			$flae=number_format($row["flae"],0,",",".")." m&#178;";
			echo "\n\t<td>".$flae."</td>"
		."\n</tr>"
	."\n</table>";
} else {
	echo "\n<p class='err'>Fehler! Kein Treffer bei gml_id=".$gmlid."</p>";
}

echo "\n<p class='nwlink'>"
	."\n\t<a href='javascript:imFenster(\"alkisbaurecht.php?gkz=".$gkz."&amp;gmlid=".$gmlid."\")' "
	."' title='Bau-, Raum- oder Bodenordnungsrecht'>Weitere Auskunft <img src='ico/Gericht.png' width='16' height='16' alt=''></a>"
."\n</p>";

?>

</body>
</html>
