<?php
/*	Navigation mit ALKIS-Daten im Mapbender 2 - Teil Adressen-Suche 
	Datenbank-Struktur aus dem norGIS-ALKIS-Importer.
Version vom 
	2016-02-11 Version für norGIS-ALKIS-Import aus Version Classic abgeleitet.
	2016-03-02 $gemeinde auf feste Länge korrigieren
	2016-11-28 Gemeinsam genutzte Datenbanken ermöglichen
	2018-02-15 Strikte Input-Validation der Parameter
	2018-11-12 Full-Schema
ToDo:
	-	Gruppierung (mit Zeile) der Straßenliste nach Gemeinde
	-	Eingabe aus "Balken" von Buchauskunft "Lage" zulassen: Numerisch: Gem-Str-Haus-lfd
		-- lfd (Nebengebäude) als Untergliederung der geklickten Haus-Nr anzeigen
		Analog zur Zerlegung des FS-Kennz in _fls
	-	Mouse-Over in Straßenliste soll Position zeigen.
		Dazu in der DB eine Tabelle mit Koordinate zum Straßenschlüssel aufbauen. 
*/

$cntget = extract($_GET);

// Input-Validation
// Parameter aus Konfiguration oder Links sollten keine "Eingabefehler" enthalten.
// Entweder falsche Konfiguration oder Hack. Programm sofort abbrechen.
if (!isset($gkz) or !preg_match('#^[0-9]{3}$#', $gkz)) {die("Eingabefehler gkz");} // Mandant, muss!

if(isset($epsg)) { // SRS, kann. Wenn als Parameter ...
	if (preg_match('#^EPSG:[0-9]{4,5}$#', $epsg)) { // Prefix aus MB-Mapframe
		$epsg = str_replace("EPSG:", "" , $epsg);
	} elseif (!preg_match('#^[0-9]{4,5}$#', $epsg)) { // nur die Nummer
		die("Eingabefehler epsg");
	}
} else { // Nichts übergeben
	$epsg=$gui_epsg; // dann default aus Conf
}

// Filter-Parameter für Gemeindenummer in Kreis-Datenbank aus der Mapbender GUI-Konfiguration
if (isset($gemeinde) AND trim($gemeinde, '0') != "") {
	if (!preg_match('#^[0-9]{1,3}$#', $gemeinde)) {die("Eingabefehler Gemeinde");}
	$gemeinde=str_pad($gemeinde, 3, "0", STR_PAD_LEFT);
	if(strpos($gemeinde, ",") === false) {
		$gfilter = 1; // Einzelwert
	} else {
		$gfilter = 2; // Liste
	}
} else { // Kein Filter
	$gemeinde = "";
	$gfilter = 0;
}
// // http://skrzmaptest01.asp.krzwan.de/mapbender/nav/alkisnav_adr.php?gkz=210&gemeinde=004&epsg=25832&str_schl=0575800401798
if (isset($str_schl) ) {
	if (!preg_match('#^[0-9]{4,13}$#', $str_schl)) {die("Eingabefehler Stra&zlig;enschl&uuml;ssel");}
} else {
	$str_schl="";
}

include("../../conf/alkisnav_conf.php");
include("alkisnav_fkt.php"); // Funktionen
$con = pg_connect ($dbconn) or die ("Fehler bei der Verbindung zur Datenbank ".$dbpre.$dbgkz);

echo <<<END
<!doctype html>
<html lang="de">
<head>
	<meta http-equiv="cache-control" content="no-cache">
	<meta http-equiv="pragma" content="no-cache">
	<meta http-equiv="expires" content="0">
	<meta charset="utf-8">
	<title>ALKIS-Suche Adressen</title>
	<link rel="stylesheet" type="text/css" href="alkisnav.css">
	<script type='text/javascript'>
		function transtitle (trans) { // Titel der letzten Transaktion anzeigen
			document.getElementById('transaktiontitle').innerHTML = trans;
		}
		function imFenster(dieURL) {
			var link = encodeURI(dieURL);
			window.open(link,'','left=10,top=10,width=620,height=800,resizable=yes,menubar=no,toolbar=no,location=no,status=no,scrollbars=yes');
		}
	</script>
</head>
<body>
<a href='javascript:history.back()'>
	<img src="ico/zurueck.png" width="16" height="16" alt="&lt;&lt;" title="zur&uuml;ck">
</a>&nbsp;
<span title='zuletzt ausgef&uuml;hrte Aktion'>
	<dfn class='title' id='transaktiontitle'></dfn>
</span>

END;

function suchStrName() {
	// Strassen nach Name(-nsanfang)
	global $street, $scalestr, $str_schl, $gkz, $gemeinde, $epsg, $gfilter, $debug, $auskpath;
	//if ($debug > 1) {echo "<p class='dbg'>function suchStrName()<p>";} // Ablauf-Verfolgung

	$linelimit=120;  // -> in Conf?
	$str_schl=""; // Loeschen vorh. Ergebnis
	preg_match("/^(\D+)(\d*)(\D*)/",$street,$matches); # 4 matches name/nr/zusatz 
	//echo "match: 1='".$matches[1]."', 2='".$matches[2]."', 3='".$matches[3]."'";
	$matches[1] = preg_replace("/strasse/i","str", $matches[1]);
	$matches[1] = preg_replace("/str\./i","str", $matches[1]); 

	if(preg_match("/\*/",$matches[1])){
		$match=trim(preg_replace("/\*/i","%", strtoupper($matches[1])));
	} else {
		$match=trim($matches[1])."%";
	}

	$sql ="SELECT g.gemeinde, g.bezeichnung AS gemname, k.gml_id, k.bezeichnung, k.schluesselgesamt, k.lage "
	."FROM ax_lagebezeichnungkatalogeintrag k "
	."JOIN ax_gemeinde g ON k.land=g.land AND k.regierungsbezirk=g.regierungsbezirk AND k.kreis=g.kreis AND k.gemeinde=g.gemeinde "
	."WHERE k.bezeichnung ILIKE $1 AND k.endet IS NULL AND g.endet IS NULL ";

	switch ($gfilter) {
		case 1: // Einzelwert
			$sql.="AND k.gemeinde='".$gemeinde."' ";
			break;
		case 2: // Liste
			$sql.="AND k.gemeinde in (".str_replace(",", "','", $gemeinde).") ";
			break;
		default: // kein Filter
			break;
	}
	$sql.="ORDER BY g.bezeichnung, k.bezeichnung, k.lage LIMIT $2 ;";
 	$v=array($match,$linelimit);
	$res=pg_prepare("", $sql);
	$res=pg_execute("", $v);

	if (!$res) {
		echo "\n<p class='err'>Fehler bei Name</p>";
		if ($debug > 2) {echo "<p class='dbg'>SQL = '".$sql."'<p>";}
		return;
	}

	$cnt = 0;
	$gwgem="";
	while($row = pg_fetch_array($res)) {
		$gemname=$row["gemname"];
		$gemnr=$row["gemeinde"] ;
		if ($gwgem != $gemname) {
			if ($gfilter != 1) {
				zeile_gemeinde($gemnr, $gemname, false); // ToDo: aber ohne Link oder Link verarbeiten können
			}
			$gwgem=$gemname;
		}
		$gkey=$row["schluesselgesamt"]; // Land-RegBez-Kreis-Gem-Strasse - für weitere Suche
		$skey=$row["lage"]; // Nur Str.-schl. daraus
		$kgml=$row["gml_id"]; // ID von Katalog
		$sname=htmlentities($row["bezeichnung"], ENT_QUOTES, "UTF-8");
		zeile_strasse ($gkey, $skey, $sname, $kgml, $gemname);
		$cnt++;
	}

	if($cnt == 0) {
		echo "\n<p class='anz'>Keine Stra&szlig;e</p>";
		//if ($debug > 2) {echo "<p class='err'>SQL = '".$sql."'<br>$1 = '".$match."'<p>";}
	} elseif($cnt == 1) { // Eindeutig
		$str_schl=$gkey; // dann gleich weiter
	} elseif($cnt >= $linelimit) {
		echo "\n<p class='anz'>".$cnt." Stra&szlig;en ... und weitere</p>";			
	} elseif ($cnt > 1) {
		echo "\n<p class='anz'>".$cnt." Stra&szlig;en</p>";	
	}
	return;
}

function suchStrKey() { // Strassen nach num. Schluessel
	global $street, $scalestr, $str_schl, $gkz, $gemeinde, $epsg, $gfilter, $debug, $auskpath;
	//if ($debug > 1) {echo "<p class='dbg'>function suchStrKey()<p>";} // Ablauf-Verfolgung
	$linelimit=60;
	if(preg_match("/\*/",$street)) {
		$match=trim(preg_replace("/\*/i","%",$street));
		// fuehrende Nullen eingeben oder fuehrende Wildcard
	} else {
		$match=str_pad($street, 5, "0", STR_PAD_LEFT); // "Wie eine Zahl" verarbeiten 
	}
	$sql ="SELECT g.bezeichnung AS gemname, k.gml_id, k.bezeichnung, k.schluesselgesamt, k.lage "
	."FROM ax_lagebezeichnungkatalogeintrag as k "
	."JOIN ax_gemeinde g ON k.land=g.land AND k.regierungsbezirk=g.regierungsbezirk AND k.kreis=g.kreis AND k.gemeinde=g.gemeinde "
	."WHERE k.lage LIKE $1 AND k.endet IS NULL AND g.endet IS NULL ";
	switch ($gfilter) {
		case 1: // Einzelwert
			$sql.="AND k.gemeinde='".$gemeinde."' ";
			break;
		case 2: // Liste
			$sql.="AND k.gemeinde in ('".str_replace(",", "','", $gemeinde)."') ";
			break;
	}
	$sql.="ORDER BY k.lage, k.bezeichnung LIMIT $2 ;";

 	$v=array($match,$linelimit);
	$res=pg_prepare("", $sql);
	$res=pg_execute("", $v);
	if (!$res) {
		echo "\n<p class='err'>Fehler bei Schl&uuml;ssel</p>";
		return;
	}
	$cnt = 0;
	while($row = pg_fetch_array($res)) {
		$sname=htmlentities($row["bezeichnung"], ENT_QUOTES, "UTF-8");		
		$gkey=$row["schluesselgesamt"];
		$gemname=htmlentities($row["gemname"], ENT_QUOTES, "UTF-8");
		$skey=$row["lage"];
		$kgml=$row["gml_id"]; // ID von Katalog	
		zeile_strasse ($gkey, $skey, $sname, $kgml, $gemname);
		$cnt++;
	}
	if($cnt == 0) {
		echo "\n<p class='anz'>Keine Stra&szlig;e mit Schl&uuml;ssel ".$match."</p>";
	} elseif($cnt == 1) { // Eindeutig
		$str_schl=$gkey; // dann gleich weiter
	} elseif ($cnt >= $linelimit) {
		echo "\n<p>".$cnt." Stra&szlig;en ... und weitere</p>";			
	} elseif ($cnt > 1) {
		echo "\n<p class='anz'>".$cnt." Stra&szlig;en</p>";	
	}	
	return;
}

function suchHausZurStr($showParent) { // Haeuser zu einer Strasse
	global $str_schl, $gkz, $scalestr, $scalehs, $epsg, $gemeinde, $epsg, $gfilter, $debug, $auskpath;
	//if ($debug > 1) {echo "<p class='dbg'>function suchHausZurStr()<p>";} // Ablauf-Verfolgung

	// Head
	// Strasse zum Strassenschluessel
	$sql ="SELECT g.bezeichnung AS gemname, k.gml_id AS kgml, k.bezeichnung, k.land, k.regierungsbezirk, k.kreis, k.gemeinde, k.lage "
	."FROM ax_lagebezeichnungkatalogeintrag as k "
	."JOIN ax_gemeinde g ON k.land=g.land AND k.regierungsbezirk=g.regierungsbezirk AND k.kreis=g.kreis AND k.gemeinde=g.gemeinde "
	."WHERE k.schluesselgesamt = $1 AND k.endet IS NULL AND g.endet IS NULL LIMIT 1"; 

  	$v=array($str_schl);	// Schluessel-Gesamt ..
	$res=pg_prepare("", $sql);
	$res=pg_execute("", $v);
	if (!$res) {
		echo "\n<p class='err'>Fehler bei Name zum Stra&szlig;enschl&uuml;ssel</p>";
		if ($debug > 2) {echo "<p class='dbg'>SQL = '".$sql."'<p>";}
		return;
	}

	if($row = pg_fetch_array($res)) { // .. gefunden
		$kgml=$row["kgml"]; // ID aus Katalog
		$sname=$row["bezeichnung"];
		$land =$row["land"];	// Einzel-Felder für JOIN _lagebezeichnung_
		$regb =$row["regierungsbezirk"];
		$kreis=$row["kreis"];
		$gemnd=$row["gemeinde"];
		$nr=$row["lage"];
		$gemname=htmlentities($row["gemname"], ENT_QUOTES, "UTF-8");
		if ($showParent) {
			// EINE Koordinate zur Strasse besorgen
			// ax_Flurstueck >zeigtAuf> ax_LagebezeichnungOhneHausnummer
			$sqlko ="SELECT ";
			if($epsg == "25832") { // Transform nicht notwendig
				$sqlko.="st_x(st_Centroid(f.wkb_geometry)) AS x, st_y(st_Centroid(f.wkb_geometry)) AS y ";
			} else {
				$sqlko.="st_x(st_transform(st_Centroid(f.wkb_geometry), ".$epsg.")) AS x, "
				."st_y(st_transform(st_Centroid(f.wkb_geometry), ".$epsg.")) AS y ";
			}
			$sqlko.="FROM ax_lagebezeichnungohnehausnummer o JOIN ax_flurstueck f ON o.gml_id=ANY(f.zeigtauf) "
			."WHERE o.land= $1 AND o.regierungsbezirk= $2 AND o.kreis= $3 AND o.gemeinde= $4 AND o.lage= $5 LIMIT 1;"; // die erstbeste Koordinate

			$v=array($land,$regb,$kreis,$gemnd,$nr);
			$resko=pg_prepare("", $sqlko);
			$resko=pg_execute("", $v);
			if ($resko) {
				$rowko=pg_fetch_array($resko); 
				$x=$rowko["x"];
				$y=$rowko["y"];
			} else {		
				echo "\n<p class='err'>Fehler bei Koordinate zur Stra&szlig;e</p>";
			}

			// "function zeile_strasse()" hier nicht verwendbar. Zusätzlicher Map-Link mit Koordinate 
			// Icon -> Buchnachweis
			echo "\n\t<br><a href='javascript:imFenster(\"".$auskpath."alkisstrasse.php?gkz=".$gkz."&amp;gmlid=".$kgml."\")'>";
				echo "\n\t\t<img class='nwlink' src='ico/Lage_mit_Haus.png' width='16' height='16' alt='STR' title='Buchauskunft zur Stra&szlig;e'>";
			echo "\n\t</a>";

			if ($x > 0) { // Koord. bekommen?
				echo "\n\t<a title='Positionieren 1:".$scalestr."' href='javascript:" // mit Link
						."transtitle(\"auf Stra&szlig;e positioniert\"); "
						."parent.parent.parent.mb_repaintScale(\"mapframe1\",".$x.",".$y.",".$scalestr."); "
						."parent.parent.showHighlight(".$x.",".$y."); "
						."' " // end href
					."\n\t\tonmouseover='parent.parent.showHighlight(" .$x. "," .$y. ")' "
					."\n\t\tonmouseout='parent.parent.hideHighlight()'"
				.">\n\t\t".$sname." (".$nr.")\n\t</a>";
			} else { // keine Koord. gefunden
				echo $sname." (".$nr.")"; // nur Anzeige, ohne Link
			}
			switch ($gfilter) {
				case 0: // Kein Filter
					echo " in ".$gemname;
					break;
				case 2: // Liste
					echo " in ".$gemname;
					break;
			}			
		}
		echo "\n<hr>";

		// Body
		// Haeuser zum Strassenschluessel
		$sql="SELECT min(replace(h.hausnummer,' ','')) AS hsnr, ";
		if($epsg == "25832") { // Transform nicht notwendig
			$sql.="avg (st_x(p.wkb_geometry)) AS x, avg (st_y(p.wkb_geometry)) AS y ";		
		} else {  
			$sql.="avg (st_x(st_transform(p.wkb_geometry,".$epsg."))) AS x, "
			."avg (st_y(st_transform(p.wkb_geometry,".$epsg."))) AS y ";		
		}
		$sql.="FROM ap_pto p JOIN ax_lagebezeichnungmithausnummer h ON h.gml_id=ANY(p.dientzurdarstellungvon) "
		."WHERE p.art='HNR' AND h.land= $1 AND h.regierungsbezirk= $2 AND h.kreis= $3 AND h.gemeinde= $4 AND h.lage= $5 "
		."AND p.endet IS NULL AND h.endet IS NULL "
		."GROUP BY lpad(split_part(hausnummer,' ',1), 4, '0'), split_part(hausnummer,' ',2) "
		."ORDER BY lpad(split_part(hausnummer,' ',1), 4, '0'), split_part(hausnummer,' ',2);";
		// Problem: In der Datenbank gibt es mehrere hausnummern-Koordinaten für verschiedene 
		// Maßstäbe der Kartendarstellung
		// Diese sollten nicht mehrfach gelistet werden. Für die Positionierung "irgendeine" nehmen.
		// Lösung: über GROUP BY in SQL. Alternative Lösungen wären: 
		//  1. Gruppenwechsel bei Abarbeitung des Result
		//  2. Subquery mit LIMIT 1 statt JOIN
		//  3. Koordinate aus dem Mittelpunkt der Gebäude-Geometrie verwenden statt aus dem Präsentationsobjekt der Hausnummer

 		$v=array($land,$regb,$kreis,$gemnd,$nr);
		$resh=pg_prepare("", $sql);
		$resh=pg_execute("", $v);
		if (!$resh) {
			echo "\n<p class='err'>Fehler bei H&auml;user zum Stra&szlig;enschl&uuml;ssel</p>";
			if ($debug > 2) {echo "<p class='dbg'>SQL='".$sql."'<br>Array=".$v."</p>";}
			return;
		}

		$cnt=0;
		$count=0;
		echo "\n<table>";
		while($rowh = pg_fetch_array($resh)) { // mehrere HsNr je Zeile
			if($count == 0) {echo "\n<tr>";}	
			$hsnr=$rowh["hsnr"];
			$x=$rowh["x"];
			$y=$rowh["y"];
			echo "\n\t<td class='hsnr'>"
				."<a href='javascript:transtitle(\"auf Haus ".$hsnr." positioniert\"); "
					."parent.parent.parent.mb_repaintScale(\"mapframe1\",".$x.",".$y.",".$scalehs."); "
					."parent.parent.showHighlight(".$x.",".$y.");' "
					."onmouseover='parent.parent.showHighlight(".$x.",".$y.")' "
					."onmouseout='parent.parent.hideHighlight()"
				."'>".$hsnr."</a>"
			."</td>";
			$cnt++;
			$count++;
			if($count == 7) { // Max. Hausnummern je Zeile (Test: 3stellige HsNr mit Zusatz)
				echo "\n</tr>";
				$count = 0;
			}
		}
		if($count > 0) {echo "\n</tr>";}
		echo "\n</table>";
		if ($cnt > 1) {
			echo "\n<p class='anz'>".$cnt." Hausnummern</p>";
		}
	} else {
		echo "\n<p class='anz'>Keine Stra&szlig;e</p>";
	}
	return;
}
// ===========
// Start hier!
// ===========

// Validation: Eingabefeld aus Formular.
// Suchbegriff (Straßenname) oder Straßenschlüssel.
// Hier formatierte Meldung und html-Foot.
if (isset($street) and $street != "" ) {
	if (!preg_match('#^[0-9a-zA-ZäöüÄÖÜß* -.]{1,25}$#', $street)) {
		echo "<p class='err'>Eingabe unzul&auml;ssig!</p>";
		$street="";
	}
} else {
	$street="";
}

if ($str_schl != "") { // aus Link
	$trans="Hausnummern zur Stra&szlig;e";
	suchHausZurStr(true);
} elseif($street != "") { // Eingabe in Formular
	if (trim($street, "*,0..9, ") == "") { // Zahl, ggf. mit Wildcard
		$trans="Suche Stra&szlig;enschl&uuml;ssel \"".$street."\"";
		suchStrKey();
	} else { // Name suchen
		$trans="Suche Stra&szlig;enname \"".$street."*\"";
		suchStrName();
	}
	if($str_schl != "") { // EINdeutiges Ergebnis
		$trans="1 Stra&szlig;e gefunden, Hausnummern";
		suchHausZurStr(false);
	}
} else {
	$trans="keine Eingabe";
}

// Nach Durchlaufen des PHP-Scriptes die zuletzt ausgeführte Transaktion 
//  im Kopf des Ergebnisrahmens anzeigen.
// Dazu die im HTML-Header definierte Javascript-Function benutzen.
// Alternativ wird auch aus dem Javascript "positionieren Karte" dieser Titel gesetzt.
echo "
<script type='text/javascript'>
	transtitle ('".$trans."') ; 
</script>";

?>

</body>
</html>
